package com.hywa.banktest.bankframework.schedule;

import java.io.File;
import java.util.List;

import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import com.hywa.banktest.bankframework.pack.Dlmaps;
import com.hywa.banktest.bankframework.pack.OUT;
import com.hywa.banktest.bankframework.pack.TransVo;
import com.hywa.banktest.bankframework.utils.FtpUtil;
import com.hywa.banktest.bankframework.utils.TransUtils;
import com.hywa.banktest.model.OrderInfo;
import com.hywa.banktest.service.OrderInfoService;

@Slf4j
@Component("bankRequsetProcess")
public class BankRequsetProcess {

	@Autowired
	FtpUtil ftpUtil;
	@Value("${money.clear.file.base.retdir}")
	private String baseDir;
	@Value("${money.clear.ftp.retDir}")
	private String ftpDownloadDir;		
			
	@Autowired
	OrderInfoService orderInfoService;
	
	public Dlmaps process(Dlmaps bankDlmaps) {
		String retFile = bankDlmaps.getIN().getTRADE_FILE_RET();
		ftpUtil.downloadFile(ftpDownloadDir, retFile, baseDir + File.separator + retFile);
		
		List<TransVo> trans = TransUtils.fileToTransVo(baseDir, retFile);
		try{
			for (TransVo tran : trans) {
				OrderInfo orderInfo = new OrderInfo();
				orderInfo.setId(tran.getOrderNo());
				orderInfo.setTransRetFile(retFile);
				orderInfo.set("transCode", tran.getReturnCode());
				orderInfo.set("transMsg", tran.getReturnMsg());
				if (tran.getBankDealState().equals("0000")) {
					orderInfo.setENABLE(3);
				}else {
					orderInfo.setENABLE(5);
				}
				orderInfoService.update(orderInfo);
			}
		}catch (Exception e){
			log.info("请求成功，但是修改的时候发生了错误{}",e);
		}
		OUT out = new OUT();
		out.setRET_CODE("0000");
		out.setMSG("success");
		bankDlmaps.setOUT(out);
		return bankDlmaps;
	}

}
