package com.hywa.banktest.bankframework.utils;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.MalformedURLException;
import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPFile;
import org.apache.commons.net.ftp.FTPReply;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import templateservice.FileKit;


@Component
public class FtpUtil {
	
	@Value("${bank.post.ftp.address}")
    public String hostname;
	@Value("${bank.post.ftp.port}")
    public Integer port;
	@Value("${bank.post.ftp.username}")
    public String username;
	@Value("${bank.post.ftp.password}")
    public String password;
	
	private FTPClient getFtpClient() {
		FTPClient ftpClient = new FTPClient();
		ftpClient.setControlEncoding("utf-8");
		try {
			ftpClient.connect(hostname, port);
			ftpClient.login(username, password);
			int replyCode = ftpClient.getReplyCode();
			if (!FTPReply.isPositiveCompletion(replyCode)) {
				throw new RuntimeException("登陆ftp服务器失败");
			}
			return ftpClient;
		} catch (MalformedURLException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return null;
	}
	
	private void closeFtpClient(FTPClient ftpClient) {
		if (null != ftpClient) {
			try {
				ftpClient.logout();
			} catch (IOException e) {
			}
			if (ftpClient.isConnected()) {
				try {
					ftpClient.disconnect();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
	}

	//public FtpUtil(String hostname, Integer port, String username, String password, FTPClient ftpClient) {
		//super();
		//this.hostname = hostname;
		//this.port = port;
		//this.username = username;
		//this.password = password;
	//}

	public FtpUtil() {
		super();
	}

	/**
	 * 上传文件
	 * @param pathname ftp服务保存地址
	 * @param fileName 上传到ftp的文件名
	 * @param localFilePath 待上传文件的名称（绝对地址）
	 * @return
	 */
	public void uploadFile(String pathname, String fileName, String localFilePath) {
		InputStream inputStream = null;
		FTPClient ftpClient = null;
		try {
			ftpClient = getFtpClient();
			inputStream = new FileInputStream(new File(localFilePath));
			ftpClient.setFileType(FTPClient.BINARY_FILE_TYPE);
			ftpClient.makeDirectory(pathname);
			ftpClient.changeWorkingDirectory(pathname);
			ftpClient.storeFile(fileName, inputStream);
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			closeFtpClient(ftpClient);
			if (null != inputStream) {
				try {
					inputStream.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
	}

	/**
	 * 上传文件
	 * 
	 * @param pathname 		ftp服务保存地址
	 * @param fileName  		上传到ftp的文件名
	 * @param inputStream 	输入文件流
	 * @return
	 */
	public void uploadFile(String pathname, String fileName, InputStream inputStream) {
		FTPClient ftpClient = null;
		try {
			ftpClient = getFtpClient();
			ftpClient.setFileType(FTPClient.BINARY_FILE_TYPE);
			ftpClient.makeDirectory(pathname);
			ftpClient.changeWorkingDirectory(pathname);
			ftpClient.storeFile(fileName, inputStream);
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			closeFtpClient(ftpClient);
			if (null != inputStream) {
				try {
					inputStream.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
	}

	public boolean existFile(String ftpDir, String ftpFileName) {
		FTPClient ftpClient = getFtpClient();
		boolean flag = false;
		try {
			ftpClient.changeWorkingDirectory(ftpDir);
		} catch (IOException e2) {
			e2.printStackTrace();
		}
		FTPFile[] ftpFileArr = null;
		try {
			ftpFileArr = ftpClient.listFiles(ftpFileName);
		} catch (IOException e2) {
			e2.printStackTrace();
		}
		if (null == ftpFileArr || ftpFileArr.length > 0) {
			flag = true;
		}
		closeFtpClient(ftpClient);
		return flag;
	}

	/**
	 * * 下载文件 
	 * @param ftpDir FTP服务器文件目录 
	 * @param ftpFileName 文件名称 
	 * @param localpath 下载后的文件路径 
	 * @return
	 */
	public void downloadFile(String ftpDir, String ftpFileName, String localpath) {
		OutputStream os = null;
		FTPClient ftpClient = null;
		try {
			ftpClient = getFtpClient();
			ftpClient.changeWorkingDirectory(ftpDir);
			FTPFile[] ftpFiles = ftpClient.listFiles();
			for (FTPFile file : ftpFiles) {
				if (ftpFileName.equalsIgnoreCase(file.getName())) {
					File localFile = new File(localpath);
					FileKit.createFile(localpath);
					os = new FileOutputStream(localFile);
					ftpClient.retrieveFile(file.getName(), os);
					os.close();
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			closeFtpClient(ftpClient);
			if (null != os) {
				try {
					os.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
	}

	public void deleteFile(String ftpDir, String filename) {
		FTPClient ftpClient = null;
		try {
			ftpClient = getFtpClient();
			ftpClient.changeWorkingDirectory(ftpDir);
			ftpClient.dele(filename);
		} catch (Exception e) {
			e.printStackTrace();
		} 
		finally {
			closeFtpClient(ftpClient);
		}
	}
}
