package com.hywa.banktest.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.hywa.banktest.bankframework.pack.Dlmaps;
import com.hywa.banktest.bankframework.pack.TransVo;
import com.hywa.banktest.bankframework.service.BankServiceImpl;
import com.hywa.banktest.bankframework.utils.httpclient.EasyHttpUtils;
import com.hywa.banktest.common.ValidateSignUtils;
import com.hywa.banktest.dao.ClearingRecordDetailMapper;
import com.hywa.banktest.dao.OrderInfoMapper;
import com.hywa.banktest.entity.*;
import com.hywa.banktest.kit.SqlKit;
import com.hywa.banktest.model.OrderInfo;
import com.hywa.banktest.service.ClearingRecordService;
import com.hywa.banktest.service.OrderInfoService;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

import com.hywa.banktest.service.PaymentLogService;
import com.hywa.banktest.service.PaymentRecordService;
import com.jfinal.kit.JsonKit;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;

@Slf4j
@Service
public class OrderInfoServiceImpl extends BaseServiceImpl<OrderInfo> implements OrderInfoService {

	@Resource
	private OrderInfoMapper orderInfoMapper;

	@Resource
	private ClearingRecordDetailMapper  clearingRecordDetailMapper;

	@Autowired
	private PaymentRecordService paymentRecordService;
	@Autowired
	BankServiceImpl bankService;
	@Autowired
	private PaymentLogService paymentLogService;
	@Autowired
	private ClearingRecordService clearingRecordService;

	@Value("${trans.post.url}")
	private String targetUrl;


	@Autowired
	private ValidateSignUtils validateSignUtils;
	@Override
	public List<OrderInfo> getTransOrderInfo(OrderInfo queryModel) {
		
		List<Object> params = new ArrayList<>();
		StringBuffer sql = new StringBuffer(super.getSql("orderInfo.getTransOrderInfo"));
		SqlKit.appendEqualCondition(sql, queryModel, params,"order_info.is_send", "order_info.ENABLE", "farmer.BANK_TYPE", "farmer.bank_validate");
		if (null != queryModel.getCreateDate()) {
			sql.append(" and ").append("order_info.CREATE_DATE").append(" >=? ");
			params.add(queryModel.getCreateDate());
		}
		log.info("assembly sql=================>{}",sql.toString());
		return super.find(sql.toString(), params.toArray());
	}

	@Override
	public List<Map> queryTransFarmerOrderInfo(Map map) {
		return orderInfoMapper.queryTransFarmerOrderInfo(map);
	}

	/**
	 * 通过farmerId更新订单信息
	 *
	 * @param params
	 * @return
	 */
	@Override
	public int updateOrderInfoByFarmer(Map params) {
		return orderInfoMapper.updateOrderStatusByFarmerId(params);
	}

	/**
	 * 查询有订单记录农户信息，为验证测试打款使用
	 *
	 * @return
	 */
	@Override
	public List<Map> queryTransFarmerForTest() {
		return orderInfoMapper.queryTransFarmerForTest();
	}

	/**
	 * 处理清款记录明细
	 *
	 * @param recordId
	 * @param companyType
	 * @return
	 */
	@Transactional(rollbackFor = Exception.class)
	@Override
	public int dealClearingRecordByRid(String recordId, String companyType) {

		if(null!= recordId && !"".equals(recordId)){
			int count=1;
			try {
				//TODO 查询清款记录明细
				List<ClearingRecordDetail> list = clearingRecordDetailMapper.getClearingRecordDetailFkId(recordId);
				if(null!=list && list.size()>0){
					String code ="2001";
					if (!"1".equals(companyType)){
						code="2002";
					}
					clearingRecordDetailMapper.updateClearingRecordCompanyId(recordId,code);

					//TODO 更新清款记录明细状态为转款中
					ClearingRecordDetail bean = new ClearingRecordDetail();
					bean.setFkClearingId(recordId);
					bean.setClearingState(2);
					clearingRecordDetailMapper.updateClearingRecordDetailInfoByRid(bean);
					//TODO 转换为资金清算服务的列表信息
					log.info("开始转换为资金清算服务的列表信息");
					List<TransVo> transVos = JSON.parseArray(JsonKit.toJson(list), TransVo.class);
					log.info("transVos====================>{}",transVos);
					Dlmaps moneyClearResult = bankService.moneyClearing(transVos,companyType);
					log.info("moneyClearResult====================>{}",moneyClearResult);
					if (!moneyClearResult.getOUT().getRET_CODE().equals("0000")) {
						this.updateAllClearingDetailOrderInfo(transVos);
						count=0;
					}
					PaymentLog paymentLog = new PaymentLog();
					paymentLog.setResponesStatus(moneyClearResult.getOUT().getRET_CODE());
					paymentLog.setResponesMsg(moneyClearResult.getOUT().getMSG());
					paymentLog.setPaymentNum(Integer.valueOf(moneyClearResult.getIN().getTRADE_NUM()));
					paymentLog.setPaymentMoney(Float.valueOf(moneyClearResult.getIN().getTRADE_TOTAL_MONEY()));
					paymentLog.setCreateTime(new Date());
					// TODO 开始记录打款日志
					paymentLogService.insertSelectiveModel(paymentLog);
					//TODO 更新清款记录状态
					ClearingRecord record = new ClearingRecord();
					record.setId(recordId);
					record.setStatus(2);
					record.setBankClearingStatus(moneyClearResult.getOUT().getRET_CODE());
					record.setBankClearingFile(moneyClearResult.getIN().getTRADE_FILE());
					record.setBankClearingMsg(moneyClearResult.getOUT().getMSG());
					clearingRecordService.updateByIdSelective(record);
				}
			} catch (Exception e) {
				e.printStackTrace();
				log.info("发送银行打款记录异常，请联系管理员");
				count=0;
			}
			return count;
		}
		return 0;
	}
	public Dlmaps post(List<TransVo> transVos,String recordId) {

		try {
			TransRequest transRequest = new TransRequest();
			transRequest.setRecord(recordId);
			String sign = validateSignUtils.signCarbon(recordId + "|" + transVos.size());
			transRequest.setSignStr(sign);
			transRequest.setTransVos(transVos);
			String post = EasyHttpUtils.LoadInstance().post(targetUrl, (JSONObject) JSONObject.toJSON(transRequest));
			TransResponse transResponse = JSONObject.parseObject(post, TransResponse.class);
			log.info("《=====》打款返回信息为{}",JSONObject.toJSONString(transResponse));
			return transResponse.getData();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;

	}

	public static void main(String[] args) {
		String str = "";
	}

	/**
	 * 通过清款明细id，更新订单详情记录
	 *
	 * @param transVoList
	 * @return
	 */
	@Transactional(rollbackFor = Exception.class)
	@Override
	public int updateOrderInfoByClearingId(List<TransVo> transVoList) {
		int s = 0;
		s=s+ orderInfoMapper.batchUpdateOrderInfo(transVoList);

		return s;
	}

	/**
	 * 银行打款请求失败后，对订单详情的处理
	 *
	 * @param transVoList
	 * @return
	 */
	@Transactional(rollbackFor = Exception.class)
	@Override
	public int updateAllClearingDetailOrderInfo(List<TransVo> transVoList) {
		int s = 0;
		if(null!=transVoList && transVoList.size()>0){
					s=s+ orderInfoMapper.batchUpdateOrderInfoForFail(transVoList);

			return s;
		}
		return 0;
	}

	@Override
	public int updateNoOrderClearing(List<TransVo> transVoList) {
		if(null!=transVoList && transVoList.size()>0){
			return orderInfoMapper.updateAllClearDetail(transVoList);
		}
		return 0;
	}

}
